/*
 * libdigidocpp
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "ADoc.h"

#include "log.h"
#include "Signature.h"
#include "XmlConf.h"
#include "crypto/cert/X509CertStore.h"
#include "util/File.h"

#include <xsec/utils/XSECPlatformUtils.hpp>

#include <openssl/conf.h>
#include <openssl/ssl.h>

#include <sstream>

namespace digidoc
{
static std::string m_appInfo = "";
}

using namespace digidoc;

/**
 * Returns appInfo string
 */
std::string digidoc::appInfo() { return m_appInfo; }

/**
 * Return libdigidocpp version
 */
std::string digidoc::version() { return VER_STR(MAJOR_VER.MINOR_VER.RELEASE_VER.BUILD_VER); }

/**
 * Initializes libraries used in digidoc implementation.
 */
void digidoc::initialize(const std::string &appInfo)
{
    m_appInfo = appInfo;

    // Initialize OpenSSL library.
    SSL_load_error_strings();
    SSL_library_init();
    OPENSSL_config(0);

    // Initialize Apache Xerces library.
    xercesc::XMLPlatformUtils::Initialize();

    // Initialize Apache XML Security library.
    XSECPlatformUtils::Initialise();

    // Use Xml based configuration
    XmlConf::initialize();

    // Init default cert store
    X509CertStore::init();
}

/**
 * Terminates libraries used in digidoc implementation.
 */
void digidoc::terminate()
{
    X509CertStore::destroy();

    XmlConf::destroy();

    // Terminate Apache XML Security library.
    XSECPlatformUtils::Terminate();

    // Terminate Apache Xerces library.
    xercesc::XMLPlatformUtils::Terminate();

    // Try to delete temporary files created with the util::File::tempFileName, tempDirectory and createTempDirectory functions.
    util::File::deleteTempFiles();

    m_appInfo.clear();
}

/**
 * @return returns container type.
 */
ADoc::DocumentType ADoc::documentType() const
{
    return ADoc::UnknownType;
}

/**
 * Returns unique signature id
 *
 * @param unique signature id
 */
unsigned int digidoc::ADoc::newSignatureId() const
{
    unsigned int id = 0;
    while(true)
    {
        std::ostringstream sid;
        sid << "S" << id;
        bool found = false;
        for(unsigned int i = 0; i < signatureCount(); ++i)
        {
            if(getSignature(i)->getId() == sid.str())
            {
                found = true;
                break;
            }
        }
        if(!found)
            return id;
        ++id;
    }
    return id;
}
